/*
 * Implementation of the EditField class
 *
 *  HISTORY
 *  Date        Author  Remarks
 *  1996-10-17  hjb     Reformatted
 *  1999-02-04  hjb     Include errno.h for glibc2 compatibility
 */

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw3d/AsciiText.h>
#include <X11/Xaw3d/Box.h>
#include <X11/Xaw3d/Label.h>

#include "EditField.h"





void EditField::Create(Widget parent, int index, char *name,
    EditDataType edt)
{
    char s[80];         // XXX - a long name can cause buffer overflow

    dataType = edt;
    sprintf(s, "%sline%d", name, index);
    box = XtVaCreateManagedWidget(s, boxWidgetClass, parent,
        XtNorientation, XtorientHorizontal, 0);

    sprintf(s, "%slabel%d", name, index);
    label = XtVaCreateManagedWidget(s, labelWidgetClass, box,
        XtNwidth, 100, 0);

    sprintf(s, "%sinput%d", name, index);
    input = XtVaCreateManagedWidget(s, asciiTextWidgetClass, box,
        XtNwidth, 200, XtNeditType, XawtextEdit, 0);

} // Create




Boolean EditField::GetValue(char *name, void *value)
{
    char        *s;
    char        *t;
    Boolean     result = True;

    XtVaGetValues(input, name, &s, 0);

    switch (dataType)
    {
        case EDT_ULONG:
            * (unsigned long *) value = strtoul(s, &t, 10);
            if (s == t || errno != 0 || (*t != '\0' && !isspace(*t)))
                result = False;
        break;

        case EDT_DOUBLE:
            * (double *) value = strtod(s, &t);
            if (s == t || errno != 0 || (*t != '\0' && !isspace(*t)))
                result = False;
        break;

        case EDT_STRING:
            // XXX - Limit the number of chars that can be copied.
            strcpy((char *) value, s);
        break;

        default:
            //printf("Illegal data type in EditField\n");
            result = False;

    } // switch

    return result;

} // GetValue




void EditField::SetValue(char *name, void *value)
{
    char s[1024];

    switch (dataType)
    {
        case EDT_ULONG:
            sprintf(s, "%lu", * (unsigned long *) value);
        break;

        case EDT_DOUBLE:
            sprintf(s, "%20.18f", * (double *) value);
        break;

        case EDT_STRING:
            snprintf(s, sizeof s, (char *) value);
        break;

        default:
            ;
            //printf("Illegal data type in EditField\n");

    } // switch

    XtVaSetValues(input, name, s, 0);

} // SetValue

