#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

#include "netchaos.h"
#include "Rubberband.h"


/* Constant denoting the absence of the rubberband cursor */

#define NoRubber (-32767)




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

Rubberband::Rubberband(Screen *screen)
{
    SetScreen(screen);
} /* Rubberband */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Rubberband::SetScreen(Screen* screen)
{
    visible = False;

    if (DefaultDepthOfScreen(screen) == 8)
        color = 255;
    else
        color = WhitePixelOfScreen(screen);

} /* Rubberband */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Rubberband::Show(XButtonPressedEvent *event)
{
    x = event->x;
    y = event->y;
    oldx1 = NoRubber;
    oldy1 = NoRubber;

    visible = True;

} /* Show */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Rubberband::Hide()
{
    visible = False;
} /* Hide */



/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

Boolean Rubberband::Visible()
{
    return visible;
} /* Visible */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

Boolean Rubberband::HasMoved()
{
    return oldx1 != NoRubber;
} /* HasMoved */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Rubberband::GetRect(int& currx, int& curry,
    int& currwidth, int& currheight)
{
    currx = x;
    curry = y;

    currwidth = width;
    currheight = height;

} /* GetRect */




/*---------------------------------------------------------------------------
 *
 * Shows or removes the rubberband cursor by XORing a rectangle with its
 * background. The rubberband has always the same width/height ratio as the
 * image.
 *
 *---------------------------------------------------------------------------*/

void Rubberband::Draw(Widget w)
{
    Display     *display;
    Window      window;
    GC          gc;
    XGCValues   values;
    int         px, py;

    display = XtDisplay(w);
    window = XtWindow(w);
    gc = DefaultGCOfScreen(XtScreen (w));
    values.foreground = color;

    /* GCForeground is defined in X.h */
    XChangeGC(display, gc, GCForeground, &values);

    XSetFunction(XtDisplay(w), gc, GXxor);
    width = oldx1 - x;
    height = oldy1 - y;
    px = x;
    py = y;

    if (width < 0)
    {
        width = -width;
        px = oldx1;
    }

    if (height < 0)
    {
        height = -height;
        py = oldy1;
    }

    if (pic.width >= pic.height)
    {
        height = (width + height) / 2;
        width = height * pic.width / pic.height;
    }
    else
    {
        width = (width + height) / 2;
        height = width * pic.height / pic.width;
    }

    XDrawRectangle(display, window, gc,
        px, py, width + 1, height + 1);
    XSetFunction(XtDisplay(w), gc, GXcopy);
}




void Rubberband::Move(Widget w, int newx, int newy)
{
    if (visible)
    {
        if (oldx1 != NoRubber)
        {
            /* Remove old rectangle */
            Draw(w);
        }

        oldx1 = newx;
        oldy1 = newy;
        Draw(w);
    }

} // Move
