/*  Mandelbrot inner loop, optimized Pentium asm.         */                         
/*  Written by Thomas Jentzsch 1998, Terje Mathisen 1997. */                         
/*  GNU as and Netchaos adaption hjb 1999.                */                         

#define re1     0(%edi)
#define re2     8(%edi)
#define im1     16(%edi)
#define im2     24(%edi)
#define reP     32(%edi)
#define imP     40(%edi)
#define reC     48(%edi)
#define imC     56(%edi)
#define reJ     64(%edi)
#define imJ     72(%edi)
#define bailout 80(%edi)
#define width   88(%edi)
#define height  92(%edi)
#define row     96(%edi)
#define maxc    100(%edi)
#define julia   104(%edi)

#define qre     -8(%esp)

.align 4

.globl iter_i486
.globl iter_pentium_classic
.globl iter_pentium_II

iter_i486:
    movl    4(%esp), %edi       /* edi := address of Image struct */
    movl    maxc,%ecx           /* ecx is our loop counter */
    movb    julia,%al
    orb     %al,%al
    je      i4_setup_mandel

i4_setup_julia:
    fldl    imJ
    fldl    reJ
    jmp     i4_setup_perturbation

i4_setup_mandel:
    fldl    imC
    fldl    reC

i4_setup_perturbation:
    fxch
    faddl   imP
    fxch
    faddl   reP

    movl    4+bailout,%eax
    movl    %eax,%edx
    fldl    reC             /*  b   a   r   i  */
    fldl    imC             /* zim zre cre cim */

    fldl    bailout
    fldln2
    fmulp   %st(0),%st(1)   /* initial qre: always smaller than bailout */
    fstpl   qre             /* b a r i */

i4_iterloop1:
    fldl    %st(1)          /* a b a r i                 */
    fmul    %st(0),%st(0)   /* aa b a r i                */
    fldl    %st(1)          /* b aa b a r i              */
    fmul    %st(0),%st(0)   /* bb aa b a r i             */
    fldl    %st(1)          /* aa bb aa b a r i          */
    fadd    %st(5),%st(0)   /* aa+r bb aa b a r i        */
    fxch    %st(4)          /* a bb aa b aa+r r i        */
    fmulp   %st(0),%st(3)   /* bb aa ab aa+r r i         */
    fadd    %st(0),%st(1)   /* bb aa+bb ab aa+r r i      */
    fsubrp  %st(0),%st(3)   /* aa+bb ab aa-bb+r r i      */
    fxch    %st(1)          /* ab aa+bb aa-bb+r r i      */
    fadd    %st(0),%st(0)   /* 2ab aa+bb aa-bb+r r i     */
    cmp     %edx,4+qre
    ja      i4_iterexit
    fadd    %st(4),%st(0)   /* 2ab+i aa+bb aa-bb+r r i   */
    fxch    %st(1)          /* aa+bb 2ab+i aa-bb+r r i   */
    fstpl   qre             /* 2ab+i aa-bb+r r i         */
    decl    %ecx            /* count                     */
    jnz     i4_iterloop1
    fldl    %st(0)          /* s a b r i                 */

i4_iterlimit:
    fcompp                  /* b r i                     */
    subl    maxc,%ecx
    negl    %ecx
    addl    %ecx,iterations
    fldz                    /* 0 b r i                   */
    jmp     i4_iterend

i4_iterexit:
    fadd    %st(4),%st(0)   /* 2ab+i aa+bb aa-bb+r r i   */
    fxch    %st(1)          /* aa+bb 2ab+i aa-bb+r r i   */

    fldl    %st(0)          /* s s a b r i               */
    fcompp                  /* a b r i                   */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fmul    %st(0),%st(0)   /* aa b r i                  */
    fxch    %st(1)          /* b aa r i                  */
    fmul    %st(0),%st(0)   /* bb aa r i                 */
    decl    %ecx            /* count                     */
    fadd    %st(1),%st(0)   /* aa+bb aa r i              */

    subl    maxc,%ecx
    fsqrt                   /* m aa r i              */
    negl    %ecx
    fld1
    fxch
    addl    %ecx,iterations
    fyl2x                   /* log(m) aa r i    */
    fldln2
    fmulp   %st(0),%st(1)
    fld1
    fxch
    fyl2x                   /* log(log(m)) aa r i    */

i4_iterend:
    movl    %ecx,qre
    fildl   qre             /* count log(log(m)) ... */
    fsub    %st(1),%st(0)
    fxch    %st(4)

    fcompp                  /* pop stack twice */
    fcompp                  /* pop stack twice */
    ret

iter_pentium_classic:
    movl    4(%esp), %edi       /* edi := address of Image struct */
    movl    maxc,%ecx           /* ecx is our loop counter */
    movb    julia,%al
    orb     %al,%al
    je      setup_mandel

setup_julia:
    fldl    imJ
    fldl    reJ
    jmp     setup_perturbation

setup_mandel:
    fldl    imC
    fldl    reC

setup_perturbation:
    fxch
    faddl   imP
    fxch
    faddl   reP

    movl    4+bailout,%eax
    movl    %eax,%edx
    fldl    imC             /*  a   b   r   i  */
    fldl    reC             /* zre zim cre cim */

    fldl    bailout
    fldln2
    fmulp   %st(0),%st(1)   /* initial qre: always smaller than bailout */

iterloop1:
    fstpl   qre             /* a b r i                   */
    fldl    %st(0)          /* a a b r i                 */
    fxch    %st(2)          /* b a a r i                 */
    fmul    %st(0),%st(2)   /* b a ab r i                */
    cmpl    %eax,%edx
    movl    4+qre,%eax      /* For comparing using an integer instruction */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */
    decl    %ecx            /* count                     */
    ja      iterloop1
    jc      iterexit

iterlimit:
    fcompp                  /* pop stack twice */
    subl    maxc,%ecx
    negl    %ecx
    addl    %ecx,iterations
    fldz
    jmp     iterend

iterexit:
    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fmul    %st(0),%st(0)   /* aa b r i                  */
    fxch    %st(1)          /* b aa r i                  */
    fmul    %st(0),%st(0)   /* bb aa r i                 */
    subl    maxc,%ecx
    fadd    %st(1),%st(0)   /* aa+bb aa r i              */

    fsqrt                   /* m aa r i                  */
    negl    %ecx
    fld1
    fxch
    addl    %ecx,iterations
    fyl2x                   /* log(m) aa r i    */
    fldln2
    fmulp   %st(0),%st(1)
    fld1
    fxch
    fyl2x                   /* log(log(m)) aa r i    */

iterend:
    movl    %ecx,qre
    fildl   qre             /* count log(log(m)) ... */
    fsub    %st(1),%st(0)
    fxch    %st(4)

    fcompp                  /* pop stack twice */
    fcompp                  /* pop stack twice */
    ret

iter_pentium_II:
    movl    4(%esp), %edi       /* edi := address of Image struct */
    movl    maxc,%ecx           /* ecx is our loop counter */
    movb    julia,%al
    orb     %al,%al
    je      p2_setup_mandel

p2_setup_julia:
    fldl    imJ
    fldl    reJ
    jmp     p2_setup_perturbation

p2_setup_mandel:
    fldl    imC
    fldl    reC

p2_setup_perturbation:
    fxch
    faddl   imP
    fxch
    faddl   reP

    movl    4+bailout,%eax
    movl    %eax,%edx
    fldl    imC             /*  a   b   r   i  */
    fldl    reC             /* zre zim cre cim */

    fldl    bailout
    fldln2
    fmulp   %st(0),%st(1)   /* initial qre: always smaller than bailout */
    fstpl   qre             /* a b r i */

p2_iterloop1:
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    movl    4+qre,%eax      /* For comparing using an integer instruction */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */
    cmpl    %edx,%eax
    ja      p2_iterexit
    fstpl   qre             /* aa-bb+r 2ab+i r i */
    decl    %ecx            /* count                     */
    jnz     p2_iterloop1
    fldl    %st(0)          /* s a b r i */

p2_iterlimit:
    fcompp                  /* pop stack twice */
    subl    maxc,%ecx
    negl    %ecx
    addl    %ecx,iterations
    fldz                    /* 0 b r i */
    jmp     p2_iterend

p2_iterexit:
    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fldl    %st(0)          /* a a b r i */
    fxch    %st(2)          /* b a a r i */
    fmul    %st(0),%st(2)   /* b a ab r i */
    decl    %ecx            /* count                     */
    fmul    %st(0),%st(0)   /* bb a ab r i               */
    fxch    %st(2)          /* ab a bb r i               */
    fadd    %st(0),%st(0)   /* 2ab a bb r i              */
    fxch    %st(1)          /* a 2ab bb r i              */
    fmul    %st(0),%st(0)   /* aa 2ab bb r i             */
    fxch    %st(1)          /* 2ab aa bb r i             */
    fldl    %st(2)          /* bb 2ab aa bb r i          */
    fsub    %st(4),%st(0)   /* bb-r 2ab aa bb r i        */
    fxch    %st(3)          /* bb 2ab aa bb-r r i        */
    fadd    %st(2),%st(0)   /* aa+bb 2ab aa bb-r r i     */
    fxch    %st(1)          /* 2ab aa+bb aa bb-r r i     */
    fadd    %st(5),%st(0)   /* 2ab+i aa+bb aa bb-r r i   */
    fxch    %st(3)          /* bb-r aa+bb aa 2ab+i r i   */
    fsubrp  %st(0),%st(2)   /* aa+bb aa-bb+r 2ab+i r i   */

    fldl    %st(0)          /* s s a b r i */
    fcompp                  /* pop stack twice */
    fmul    %st(0),%st(0)   /* aa b r i                  */
    fxch    %st(1)          /* b aa r i                  */
    fmul    %st(0),%st(0)   /* bb aa r i                 */
    decl    %ecx
    fadd    %st(1),%st(0)   /* aa+bb aa r i              */

    subl    maxc,%ecx
    fsqrt                   /* m aa r i              */
    negl    %ecx
    fld1
    fxch
    addl    %ecx,iterations
    fyl2x                   /* log(m) aa r i    */
    fldln2
    fmulp   %st(0),%st(1)
    fld1
    fxch
    fyl2x                   /* log(log(m)) aa r i    */

p2_iterend:
    movl    %ecx,qre
    fildl   qre             /* count log(log(m)) ... */
    fsub    %st(1),%st(0)
    fxch    %st(4)

    fcompp                  /* pop stack twice */
    fcompp                  /* pop stack twice */
    ret

.globl get_cpu_type

get_cpu_type:
    /* Copied from the Linux kernel. */
    movl $3, x86
    pushfl                  # push EFLAGS
    popl %eax               # get EFLAGS
    movl %eax,%ecx          # save original EFLAGS in ecx
    xorl $0x40000,%eax      # flip AC bit in EFLAGS
    pushl %eax              # copy to EFLAGS
    popfl                   # set EFLAGS
    pushfl                  # get new EFLAGS
    popl %eax               # put it in eax
    xorl %ecx,%eax          # change in flags
    andl $0x40000,%eax      # check if AC bit changed
    je get_cpu_end
    movl $4, x86
    movl %ecx,%eax
    xorl $0x200000,%eax     # check ID flag
    pushl %eax
    popfl                   # if we are on a straight 486DX, SX, or
    pushfl                  # 487SX we cant change it
    popl %eax               # Also if we are on a Cyrix 6x86(L)
    xorl %ecx,%eax          # OTOH 6x86MXs and MIIs check OK
    andl $0x200000,%eax
    je get_cpu_end
    pushl %ecx              # restore original EFLAGS
    popfl

    movl $4, x86
    xorl    %eax,%eax
    cpuid
    movl    %ebx,x86_vendor
    movl    %edx,x86_vendor+4
    movl    %ecx,x86_vendor+8
    movl    $1,%eax
    cpuid

get_cpu_end:
    ret
