/*
 * Implementation of callback routines
 *
 *  HISTORY
 *  Date        Author  Remarks
 *  1996-10-17  hjb     Reformatted
 */

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw3d/Reports.h>

#include "dialog.h"
#include "netchaos.h"
#include "Rubberband.h"




void KeyPressCB(Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
    char            buffer[80];
    KeySym          key;
    Position        x, y;
    FileRequest     request;

    XLookupString((XKeyEvent *) event, buffer, sizeof(buffer), &key, 0);
    x = ((XKeyPressedEvent *) event)->x_root;
    y = ((XKeyPressedEvent *) event)->y_root;

    Debug(10, "Key pressed: %u ('%d'), Pointer in (%d,%d)\n",
        ((XKeyPressedEvent *) event)->keycode, (int) key,
        ((XKeyPressedEvent *) event)->x,
        ((XKeyPressedEvent *) event)->y);

    switch (key)
    {
        case 'a':
        case 'A':
            AboutPopup(x, y);
        break;

        case 'o':
        case 'O':
            OptionPopup(x, y);
        break;

        case 'p':
        case 'P':
            /* Panning */
        break;

        case 'q':
        case 'Q':
            QuitPopup(x, y);
        break;

        case 'r':
        case 'R':
            /* Read image from file */
            request.title = "Read from file";
            request.defaultName = "netchaos.ppm";
            request.mode = R_OK;
            request.function = GetImage;
            FileSelectPopup(x, y, &request);
        break;

        case 's':
        case 'S':
            /* Image size */
            SizePopup(x, y);
        break;

        case 'u':
        case 'U':
            RestoreFromStack();
        break;

        case 'w':
        case 'W':
            /* Write image to file */
            request.title = "Save to file";
            request.defaultName = "netchaos.ppm";
            request.mode = W_OK;
            request.function = WriteImage;
            FileSelectPopup(x, y, &request);
        break;

    } /* switch */

} /* KeyPressCB */





void ButtonPressCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
    Debug(10, "Button pressed: %u, Pointer in (%d,%d)\n",
        ((XButtonPressedEvent *) event)->button,
        ((XButtonPressedEvent *) event)->x,
        ((XButtonPressedEvent *) event)->y);

    if (((XButtonPressedEvent *) event)->button == 1 && !rubberband.Visible())
        rubberband.Show((XButtonPressedEvent *) event);

} /* ButtonPressCB */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void ButtonReleaseCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
    DOUBLE    temp;
    int        x, y;
    int        width, height;
    int        endx, endy;

    Debug(10, "Button released: %u, Pointer in (%d,%d)\n",
        ((XButtonPressedEvent *) event)->button,
        ((XButtonPressedEvent *) event)->x,
        ((XButtonPressedEvent *) event)->y);

    if (((XButtonReleasedEvent *) event)->button == 2 && !pic.julia)
    {
        /* Start calculation of a Julia set with mouse button 2 */
        endx = ((XButtonPressedEvent *) event)->x;
        endy = ((XButtonPressedEvent *) event)->y;
        if (endx >= 0 && endx < pic.width && endy >= 0 && endy < pic.height)
        {
            /* save old picture parameters to stack */
            SaveToStack();

            /* set new picture parameters */
            temp = (pic.re2 - pic.re1) / pic.width;
            pic.reJ = pic.re1 + endx * temp;
            temp = (pic.im2 - pic.im1) / pic.height;
            pic.imJ = pic.im1 + endy * temp;
            pic.re1 = -2.0;
            pic.re2 = 2.0;
            pic.im1 = 2.0;
            pic.im2 = -2.0;
            pic.julia = True;
            newPicture = True;
        }
    }

    if (((XButtonReleasedEvent *) event)->button == 1 && rubberband.Visible())
    {
        if (rubberband.HasMoved())
        {
            rubberband.Draw(w);    /* removes the cursor from the screen */
            rubberband.GetRect(x, y, width, height);
            endx = x + width;
            endy = y + height;

            if (endx >= 0 && endx < pic.width && endy >= 0 && endy < pic.height)
            {
                /* save old picture parameters to stack */
                SaveToStack();

                /* set new picture parameters */
                temp = (pic.re2 - pic.re1) / pic.width;

                if (endx >= x)
                {
                    pic.re2 = pic.re1 + endx * temp;
                    pic.re1 = pic.re1 + x * temp;
                }
                else
                {
                    pic.re2 = pic.re1 + x * temp;
                    pic.re1 = pic.re1 + endx * temp;
                }

                temp = (pic.im2 - pic.im1) / pic.height;

                if (endy >= y)
                {
                    pic.im2 = pic.im1 + endy * temp;
                    pic.im1 = pic.im1 + y * temp;
                }
                else
                {
                    pic.im2 = pic.im1 + y * temp;
                    pic.im1 = pic.im1 + endy * temp;
                }

                newPicture = True;
            }
        }

        rubberband.Hide();

    } /* if button 1 */

} /* ButtonReleaseCB */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void PointerMotionCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
    Debug(10, "Pointer moved, Pointer in (%d,%d)\n",
        ((XPointerMovedEvent *) event)->x,
        ((XPointerMovedEvent *) event)->y);

    rubberband.Move(w, ((XButtonPressedEvent *) event)->x,
        ((XButtonPressedEvent *) event)->y);

} /* PointerMotionCB */




/*---------------------------------------------------------------------------
 *
 * event is a XExposeEvent.
 *
 *---------------------------------------------------------------------------*/

void ExposeCB(Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
    XExposeEvent *ev = (XExposeEvent *) event;

    if (event == 0)
    {
        Debug(0, "Event pointer is 0 in ExposeCB\n");
        return;
    }

    if (clientData == 0)
    {
        Debug(0, "clientData is 0 in ExposeCB\n");
        return;
    }

    Debug(8, "Expose %d %d %d %d\n", ev->x, ev->y, ev->width, ev->height);

    XPutImage(XtDisplay(w), XtWindow(w), DefaultGCOfScreen(XtScreen(w)),
        (XImage *) clientData,
        ev->x, ev->y, ev->x, ev->y, ev->width, ev->height);

} /* ExposeCB */





void ResizeCB(Widget w, XtPointer clientData, XtPointer callData)
{
    static int entryCount = 0;
#define cd ((XawPannerReport *) callData)

    if (++entryCount <= 1)
    {
        if (cd->changed & (XawPRCanvasWidth | XawPRCanvasHeight))
        {
            Boolean modified = False;

            Debug(8, "Resize\n");
            Debug(8, "%d %d %d %d\n",
                (int) cd->slider_width,
                (int) cd->slider_height,
                (int) cd->canvas_width,
                (int) cd->canvas_height);

            if (cd->canvas_width > pic.width)
            {
                cd->canvas_width = pic.width;
                modified = True;
            }

            if (cd->canvas_height > pic.height)
            {
                cd->canvas_height = pic.height;
                modified = True;
            }

            if (modified)
            {
                XtVaSetValues(mscreen.toplevel, XtNwidth, cd->canvas_width,
                    XtNheight, cd->canvas_height, 0);
            }

            XtVaSetValues((Widget) clientData, XtNwidth, pic.width,
                XtNheight, pic.height, 0);
        }
    }

    --entryCount;

#undef cd

} /* ResizeCB */
