/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <stdio.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <sys/wait.h>

#include "netchaos.h"               /* export definitions */
#include "mdefs.h"                  /* portability        */


pid_t		child;





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Shutdown(SharedInfo *command)
{
    int shmid;
    int status;

    if (child)
    {
        kill(child, SIGTERM);
        wait(&status);
        shmid = shmget(MY_SHM_KEY, sizeof(SharedInfo), 0);
        shmdt((char *) command);
        shmctl(shmid, IPC_RMID, 0);
    }

} /* Shutdown */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

static void RemoveStaleSharedMemory()
{
    int	            shmid;
    int	            shmax;
    struct shmid_ds info;

    shmax = shmctl(0, SHM_INFO, &info);

    for (shmid = 0; shmid <= shmax; shmid++)
    {
        if (shmctl(shmid, IPC_STAT, &info) != -1)
        {
            Debug(1, "size %d = %d\n", shmid, info.shm_segsz);
            break;
        }
    }

    if (shmid <= shmax)
    {
        if (shmctl(shmid, IPC_RMID, 0) == 0)
            Debug(1, "RemoveStaleSharedMemory: successful\n");
        else
            Debug(0, "RemoveStaleSharedMemory: Could not remove\n");
    }
    else
    {
        Debug(1, "RemoveStaleSharedMemory: nothing to do\n");
    }
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void CreateChild(SharedInfo*& command)
{
    int shmid;

    RemoveStaleSharedMemory();
    shmid = shmget(MY_SHM_KEY, sizeof(SharedInfo),
        IPC_CREAT | 0666);
    if (shmid < 0)
    {
        Debug(1, "Could not create shared memory, trying to get\n");
        shmid = shmget(MY_SHM_KEY, sizeof(SharedInfo), 0);
        if (shmid < 0)
        {
            Debug(0, "Could not get shared memory\n");
            exit(1);
        }
    }

    command = (SharedInfo *) shmat(shmid, (char *) 0, 0);
    if ((int) command < 0)
    {
        shmctl(shmid, IPC_RMID, 0);
        Debug(0, "Could not attach shared memory\n");
        exit(1);
    }

    child = fork();
    if (child < 0)
    {
        shmctl(shmid, IPC_RMID, 0);
        perror("Can't fork");
        exit(1);
    }
    else if (child == 0)
    {
        Calc();
        // Never reached.
    }
}
