/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <signal.h>
#include <sys/time.h>

#include "comm.h"
#include "netchaos.h"
#include "mdefs.h"
#include "util.h"


typedef char Boolean;

typedef struct
{
    int       column, row;
    DOUBLE    reStep, imStep;
} CurrentImage;


extern LONG iterations;


static Boolean      doTest;
static SharedInfo   *command;





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

static void MCExit(int code)
{
    exit(code);
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void TermHandler(int signal)
{
    Debug(DEBUG_GEN(0), "Terminated with signal %d\n", signal);
    MCExit(1);
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/
void AlarmHandler(int whatever)
{
    doTest = True;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Calc()
{
    CurrentImage    current;
    int             cmd;
    int             imageno = 0;
    int             lineno = 0;
    int             nlines = 0;
    int             lcount;
    struct timeval  tv;
    struct timeval  tvStart;
    int             resultSize0 = 0;
    DOUBLE          *result0 = NULL;
    DOUBLE          seconds;

    enum
    {
        C_INIT,
        C_WAIT,
        C_RECV,
        C_START,
        C_READY
    } state = C_INIT;

    signal(SIGINT, TermHandler);        /* catch CTRL-C */
    signal(SIGTERM, TermHandler);       /* catch kill -TERM */

    while (True)
    {
        switch (state)
        {
            case C_INIT:
                if (!CommInitSlave())
                    MCExit(1);
                state = C_WAIT;
            break;

            case C_WAIT:
                CommWaitSlave();
                state = C_RECV;
            break;

            case C_RECV:
                command = CommGetCommand();
                if (command->cmd == CMD_QUIT)
                {
                    CommFreeBuffer();
                    MCExit(0);
                }

                memcpy(&pic, &command->pic, sizeof pic);
                cmd = command->cmd;
                imageno = command->imageno;
                lineno = command->lineno;
                nlines = command->nlines;

                /* Start receiving next command. */
                CommRecvNextSlave();

                /* Currently we only honor the CMD_START command. */
                if (cmd == CMD_START)
                    state = C_START;
                else
                    state = C_WAIT;
            break;

            case C_START:
                Debug(DEBUG_CMD(1), "Received start command for lines %d-%d\n",
                    lineno, lineno + nlines - 1);

                if (resultSize0 < pic.width * nlines + HEADER_SIZE)
                {
                    resultSize0 = pic.width * nlines + HEADER_SIZE;
                    if (!CommAllocBuffer(resultSize0))
                        MCExit(1);
                }

                if ((result0 = CommGetBuffer()) == NULL)
                {
                    DOUBLE response = RESP_ERR_NOMEM;

                    CommSendResponse(response);
                    state = C_WAIT;
                    break;
                }

                current.reStep = (pic.re2 - pic.re1) / pic.width;
                current.imStep = (pic.im2 - pic.im1) / pic.height;

                doTest = False;
                signal(SIGALRM, AlarmHandler);
                alarm(1);

                GetTime(tvStart);

                state = C_READY;
                iterations = 0;
                lcount = 0;

                while (lcount < nlines)
                {
                    pic.reC = pic.re1;
                    pic.imC = pic.im1
                            + (lineno + lcount) * (pic.im2 - pic.im1) / pic.height;
                    for (current.column = 0; current.column < pic.width; current.column++)
                    {
                        result0[lcount * pic.width + current.column + HEADER_SIZE]
                            = Iterate(&pic);
                        pic.reC += current.reStep;
                        if (doTest)
                        {
                            doTest = False;
                            signal(SIGALRM, AlarmHandler);
                            alarm(1);
                            if (CommRecvTestSlave(imageno))
                            {
                                state = C_RECV;
                                break;
                            }
                        }
                    } /* for */
                    ++lcount;
                } /* while */
            break;

            case C_READY:
                /* Calculate MFlops rate. */
                GetTime(tv);
                if (tv.tv_sec >= 0 && tv.tv_usec >= 0
                        && tvStart.tv_sec >= 0 && tvStart.tv_usec >= 0)
                {
                    seconds = tv.tv_sec - tvStart.tv_sec
                        + (tv.tv_usec - tvStart.tv_usec) / 1.0e6;
                    if (seconds >= TIME_EPSILON)
                    {
                        Debug(DEBUG_PERF(1), "%ld iterations in %.2f seconds",
                            iterations, seconds);
                        Debug(DEBUG_PERF(1), " (%.2f MFlops)\n",
                            MFLOP_FACTOR * iterations / seconds);
                    }
                }
                else
                {
                    seconds = 1000.0;
                }

                /* One block is finished. */
                result0[0] = RESP_DATA;
                result0[1] = imageno;
                result0[2] = lineno;
                result0[3] = iterations;
                result0[4] = seconds;

                CommSendResult();
                state = C_WAIT;
            break;

        } /* switch */
    } /* while */

    return;             /* never reached */

} /* Calc */
