/*
 * Global declarations
 *
 *  HISTORY
 *  Date        Author  Remarks
 *  1996-10-17  hjb     Reformatted; Added USE macro
 *  1999-02-08  hjb     Added nlines to SharedInfo struct
 *  1999-03-07  hjb     Removed handshaking fields from SharedInfo struct
 */

#ifndef NETCHAOS_H
#define NETCHAOS_H

/* Macro to avoid "Unused variable" warnings. */
#define USE(var) \
    static Boolean uses##var = &uses##var == (Boolean *) &var

#include <signal.h>
#include <unistd.h>
#include <X11/Intrinsic.h>          /* Boolean */
#include "mdefs.h"
#include "Rubberband.h"


/* Commands from client to server. */
const int CMD_START = 0;
const int CMD_QUIT = 1;
const int CMD_ABORT_LINE = 2;

/* Responses from server to client. DOUBLE for now. */
const DOUBLE RESP_DATA      = 0.0;
const DOUBLE RESP_ERR_NOMEM = 1.0;


const DOUBLE MFLOP_FACTOR = 8.0e-6;
const DOUBLE TIME_EPSILON = 0.01;

/* Picture height/MAX_LINE_FACTOR limits the number of lines per command. */
#define MAX_LINE_FACTOR 40

const int HEADER_SIZE = 5;      /* 5 DOUBLEs in response header */

const int MY_SHM_KEY = 0x3a3;

/* XXX - Where fractal images are stored */
#define frakDir "/home/hans/projekte/fractals/"


typedef struct
{
    DOUBLE        re1;
    DOUBLE        re2;
    DOUBLE        im1;
    DOUBLE        im2;
    DOUBLE        reP;                    /* re perturbation */
    DOUBLE        imP;                    /* im perturbation */
    DOUBLE        reC;                    /* current point */
    DOUBLE        imC;
    DOUBLE        reJ;                    /* point for julia fractal */
    DOUBLE        imJ;
    DOUBLE        bailout;                /* bailout value */
    int           width;                  /* width in pixels */
    int           height;                 /* height in pixels */
    int           row;                    /* row currently calculated */
    LONG          maxc;                   /* max number of iterations */
    Boolean       julia;                  /* true if julia set */
} Image;


typedef struct
{
    int         depth;          /* bits per pixel */
    int         bpp;            /* bytes per pixel */
    Display     *display;
    Colormap    colormap;
    XImage      *image;
    char        *data;          /* The image contents */
    Widget      toplevel;       /* Our toplevel widget */
    Widget      draw;           /* Widget that contains the image */
    Boolean     weight565;      /* true if green has 6 bits in 64 K colors */
} ScreenInfo;


typedef struct
{
    /* The structure of the command */
    long        cmd;                /* command number */
    long        imageno;            /* image number */
    long        lineno;             /* line number */
    long        nlines;             /* number of lines */
    Image       pic;                /* image parameters */
} SharedInfo;


extern Boolean      newPicture;
extern Image        pic;
extern Image        readpic;
extern ScreenInfo   mscreen;
extern Rubberband   rubberband;
extern pid_t        child;


extern void Debug(int level, char *format, ...);
extern void Shutdown(SharedInfo *command);
extern void CreateChild(SharedInfo*& command);
extern void Calc();
extern void Exit(int status);
extern void SaveToStack();
extern void RestoreFromStack();
extern void GetColormap(XColor *colormap);
extern void InitIteration();
extern DOUBLE Iterate(Image *image);

/* ppm.c */
extern Boolean WriteImage(const char *filename);
extern Boolean OpenImage(const char *filename);
extern Boolean ReadImage(const char *filename);
extern Boolean GetImage(const char *filename);

/* callback.c */
extern void KeyPressCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void ButtonPressCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void ButtonReleaseCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void PointerMotionCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void ExposeCB(Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void ResizeCB(Widget w, XtPointer clientData, XtPointer callData);

#endif /* NETCHAOS_H */
