#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Xaw3d/AsciiText.h>
#include <Xaw3d/Box.h>
#include <Xaw3d/Label.h>

#include "EditField.h"


void EditField :: Create (Widget parent, int index, char *name,
    EditDataType edt)
{
  char	s [100];

  dataType = edt;
  sprintf (s, "%sline%d", name, index);
  box = XtVaCreateManagedWidget (s, boxWidgetClass, parent,
    XtNorientation, XtorientHorizontal, 0);
  sprintf (s, "%slabel%d", name, index);
  label = XtVaCreateManagedWidget (s, labelWidgetClass, box,
    XtNwidth, 100, 0);
  sprintf (s, "%sinput%d", name, index);
  input = XtVaCreateManagedWidget (s, asciiTextWidgetClass, box,
    XtNwidth, 200, XtNeditType, XawtextEdit, 0);
}




Boolean EditField :: GetValue (char *name, void *value)
{
  char		*s;
  char		*t;
  Boolean	result = True;

  XtVaGetValues (input, name, &s, 0);
  switch (dataType)
  {
    case EDT_ULONG:
      * (unsigned long *) value = strtoul (s, &t, 10);
      if (s == t || errno != 0 || (*t != '\0' && !isspace (*t)))
        result = False;
    break;

    case EDT_DOUBLE:
      * (double *) value = strtod (s, &t);
      if (s == t || errno != 0 || (*t != '\0' && !isspace (*t)))
        result = False;
    break;

    case EDT_STRING:
      strcpy ((char *) value, s);
    break;

    default:
      printf ("Illegal data type in EditField\n");
      result = False;
  } /* switch */

  return result;
} /* GetValue */




void EditField :: SetValue (char *name, void *value)
{
  char s [100];

  switch (dataType)
  {
    case EDT_ULONG:
      sprintf (s, "%lu", * (unsigned long *) value);
    break;

    case EDT_DOUBLE:
      sprintf (s, "%20.18f", * (double *) value);
    break;

    case EDT_STRING:
      strcpy (s, (char *) value);
    break;

    default:
      printf ("Illegal data type in EditField\n");
  } /* switch */
  XtVaSetValues (input, name, s, 0);
} /* SetValue */

