#include <Xlib.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Shell.h>

#include <Xaw3d/Reports.h>

#include "dialog.h"
#include "mandel.h"
#include "Rubberband.h"




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void KeyPressCB (Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
  char		buffer [80];
  KeySym	key;
  Position	x, y;
  FileRequest	request;

  XLookupString ((XKeyEvent *) event, buffer, sizeof (buffer), &key, 0);
  x = ((XKeyPressedEvent *) event)->x_root;
  y = ((XKeyPressedEvent *) event)->y_root;
  Debug (1, "Key pressed: %u ('%d'), Pointer in (%d,%d)\n",
    ((XKeyPressedEvent *) event)->keycode, (int) key,
    ((XKeyPressedEvent *) event)->x,
    ((XKeyPressedEvent *) event)->y);

  switch (key)
  {
    case 'a':
    case 'A':
      AboutPopup (x, y);
    break;

    case 'o':
    case 'O':
      OptionPopup (x, y);
    break;

    case 'p':
    case 'P':
      /* Panning */
    break;

    case 'q':
    case 'Q':
      QuitPopup (x, y);
    break;

    case 'r':
    case 'R':
      /* Read image from file */
      request.title = "Read from file";
      request.defaultName = "mandel.ppm";
      request.mode = R_OK;
      request.function = GetImage;
      FileSelectPopup (x, y, &request);
    break;

    case 's':
    case 'S':
      /* Image size */
      SizePopup (x, y);
    break;

    case 'u':
    case 'U':
      RestoreFromStack ();
    break;

    case 'w':
    case 'W':
      /* Write image to file */
      request.title = "Save to file";
      request.defaultName = "mandel.ppm";
      request.mode = W_OK;
      request.function = WriteImage;
      FileSelectPopup (x, y, &request);
    break;

  } /* switch */
} /* KeyPressCB */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void ButtonPressCB (Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
  Debug (9, "Button pressed: %u, Pointer in (%d,%d)\n",
    ((XButtonPressedEvent *) event)->button,
    ((XButtonPressedEvent *) event)->x,
    ((XButtonPressedEvent *) event)->y);
  if (((XButtonPressedEvent *) event)->button == 1 && !rubberband.Visible ())
    rubberband.Show ((XButtonPressedEvent *) event);
} /* ButtonPressCB */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void ButtonReleaseCB (Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
  DOUBLE	temp;
  int		x, y;
  int		width, height;
  int		endx, endy;

  Debug (9, "Button released: %u, Pointer in (%d,%d)\n",
    ((XButtonPressedEvent *) event)->button,
    ((XButtonPressedEvent *) event)->x,
    ((XButtonPressedEvent *) event)->y);

  if (((XButtonReleasedEvent *) event)->button == 2 && !pic.julia)
  {
    /* Start calculation of a Julia set with mouse button 2 */
    endx = ((XButtonPressedEvent *) event)->x;
    endy = ((XButtonPressedEvent *) event)->y;
    if (endx >= 0 && endx < pic.width && endy >= 0 && endy < pic.height)
    {
        /* save old picture parameters to stack */
	SaveToStack ();

        /* set new picture parameters */
        temp = (pic.re2 - pic.re1) / pic.width;
	pic.reJ = pic.re1 + endx * temp;
        temp = (pic.im2 - pic.im1) / pic.height;
	pic.imJ = pic.im1 + endy * temp;
	pic.re1 = -2.0;
	pic.re2 = 2.0;
	pic.im1 = 2.0;
	pic.im2 = -2.0;
	pic.julia = True;
	/* reset pic.maxc? */
        newPicture = True;
	pic.row = 0;
    }
  }

  if (((XButtonReleasedEvent *) event)->button == 1 && rubberband.Visible ())
  {
    if (rubberband.HasMoved ())
    {
      rubberband.Draw (w);	/* removes the cursor from the screen */
      rubberband.GetRect (x, y, width, height);
      endx = x + width;
      endy = y + height;
      if (endx >= 0 && endx < pic.width && endy >= 0 && endy < pic.height)
      {
        /* save old picture parameters to stack */
	SaveToStack ();

        /* set new picture parameters */
        temp = (pic.re2 - pic.re1) / pic.width;
	if (endx >= x)
	{
          pic.re2 = pic.re1 + endx * temp;
          pic.re1 = pic.re1 + x * temp;
	}
	else
	{
          pic.re2 = pic.re1 + x * temp;
          pic.re1 = pic.re1 + endx * temp;
	}
        temp = (pic.im2 - pic.im1) / pic.height;
	if (endy >= y)
	{
          pic.im2 = pic.im1 + endy * temp;
          pic.im1 = pic.im1 + y * temp;
	}
	else
	{
          pic.im2 = pic.im1 + y * temp;
          pic.im1 = pic.im1 + endy * temp;
	}
        newPicture = True;
	pic.row = 0;
      }
    }
    rubberband.Hide ();
  } /* if button 1 */
} /* ButtonReleaseCB */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void PointerMotionCB (Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
  Debug (9, "Pointer moved, Pointer in (%d,%d)\n",
    ((XPointerMovedEvent *) event)->x,
    ((XPointerMovedEvent *) event)->y);
  rubberband.Move (w, ((XButtonPressedEvent *) event)->x,
    ((XButtonPressedEvent *) event)->y);
} /* PointerMotionCB */




/*---------------------------------------------------------------------------
 *
 * event is a XExposeEvent.
 *
 *---------------------------------------------------------------------------*/

void ExposeCB (Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
  GC gc;
  XGCValues values;
  XExposeEvent *ev = (XExposeEvent *) event;

  Debug (8, "Expose\n");
  if (event == 0)
  {
    Debug (0, "Event pointer is 0 in ExposeCB ()\n");
    return;
  }
  gc = DefaultGCOfScreen (XtScreen (w));
  XGetGCValues (XtDisplay (w), gc, 0xffffffff, &values);
  XPutImage (XtDisplay (w), XtWindow (w), gc, (XImage *) clientData,
    ev->x, ev->y, ev->x, ev->y, ev->width, ev->height);
} /* ExposeCB */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void ResizeCB (Widget w, XtPointer clientData, XtPointer callData)
{
  static int entryCount = 0;
#define cd ((XawPannerReport *) callData)

  if (++entryCount <= 1)
  {
    if (cd->changed & (XawPRCanvasWidth | XawPRCanvasHeight))
    {
      Debug (8, "Resize\n");
      Debug (8, "%d %d %d %d\n",
        (int) ((XawPannerReport *) callData)->slider_width,
        (int) ((XawPannerReport *) callData)->slider_height,
        (int) ((XawPannerReport *) callData)->canvas_width,
        (int) ((XawPannerReport *) callData)->canvas_height);
      XtVaSetValues ((Widget) clientData, XtNwidth, pic.width,
        XtNheight, pic.height, 0);
    }
  }
  --entryCount;
#undef cd
} /* ResizeCB */

