/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/wait.h>

#include "mandel.h"                 /* export-def.     */
#include "mdefs.h"                  /* portabilitaet   */


pid_t		child;




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Shutdown (void)
{
  int shmid;
  int status;

  if (child)
  {
     kill (child, SIGTERM);
     wait (&status);
     shmid = shmget (MY_SHM_KEY, sizeof (SharedInfo), 0600);
     shmdt ((char *) sharedInfo);
     while (shmctl (shmid, IPC_RMID, 0) != 0)
       ; 
  }
} /* Shutdown */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Exit (int status)
{
  Shutdown ();
  exit (status);
} /* Exit */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

static void RemoveStaleSharedMemory (void)
{
  int		shmid;

  /* We request only the minimum size, because we don't know the actual
     size of the shared memory. Hope it works. */
  shmid = shmget (MY_SHM_KEY, sizeof (SharedInfo), 0600);
  if (shmid >= 0)
  {
    if (shmctl (shmid, IPC_RMID, 0) == 0)
      Debug (0, "RemoveStaleSharedMemory: successful\n");
    else
      Debug (0, "RemoveStaleSharedMemory: Could not remove\n");
  }
  else
    Debug (0, "RemoveStaleSharedMemory: nothing to do\n");
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void CreateChild (void)
{
  int shmid;

  RemoveStaleSharedMemory ();
  shmid = shmget (MY_SHM_KEY, sizeof (SharedInfo) + sizeof (LONG) * pic.width,
    IPC_EXCL | IPC_CREAT | 0600);
  if (shmid < 0)
  {
    Debug (0, "Could not get shared memory\n");
    exit (1);
  }

  sharedInfo = (SharedInfo *) shmat (shmid, 0, 0);
  if ((int) sharedInfo < 0)
  {
    shmctl (shmid, IPC_RMID, 0);
    Debug (0, "Could not attach shared memory\n");
    exit (1);
  }

  child = fork ();
  if (child < 0)
  {
    shmctl (shmid, IPC_RMID, 0);
    Debug (0, "fork() failed\n");
    exit (1);
  }
  else if (child == 0)
  {
    /* The first parameter becomes argv [0] for the child, so we
       write its name in it. */
    if (execl ("mandel-calc", "mandel-calc", 0) < 0)
    {
      shmctl (shmid, IPC_RMID, 0);
      Debug (0, "Could not execute 'mandel-calc'\n");
      exit (1);
    }
  }
}

