/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>

#include "mandel.h"
#include "mdefs.h"


typedef char Boolean;

typedef struct
{
  int		column, row;
  DOUBLE	reStep, imStep;
} CurrentImage;


extern LONG iterations;

pid_t		child;
Image		pic;
SharedInfo	*sharedInfo;
Boolean		timeout;
FILE *f;




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void TermHandler (int signal)
{
  fprintf (f, "Terminated with signal %d\n", signal);
  fclose (f);
  shmdt ((char *) sharedInfo);
  exit (0);
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void HandshakeHandler (int sig)
{
  signal (sig, HandshakeHandler);
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void AlarmHandler (int whatever)
{
  timeout = True;
  fprintf (f, "Timeout\n");
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void VTAlarmHandler (int whatever)
{
  fprintf (f, "Iterations per second: %ld\n", iterations);
  iterations = 0;
  signal (SIGVTALRM, VTAlarmHandler);
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

int main (int argc, char **argv)
{
  int		shmid;
  int		width;
  CurrentImage	current;
  Boolean	ready;		/* True if no work to be done */
  pid_t		parent;		/* pid of parent process */
  itimerval	second;

  parent = getppid ();
  f = fopen ("/tmp/shmem.log", "w");
  fprintf (f, "%s\n", argv [0]);
  /* Allocate only the known size of SharedInfo; the rest is allocated
     later. */
  shmid = shmget (MY_SHM_KEY, sizeof (SharedInfo), 0600);
  if (shmid < 0)
  {
    fprintf (f, "Could not get shared memory\n");
    exit (1);
  }

  sharedInfo = (SharedInfo *) shmat (shmid, 0, 0);
  if ((int) sharedInfo < 0)
  {
    fprintf (f, "Could not attach shared memory\n");
    exit (1);
  }

  width = sharedInfo->pic.width;
  shmdt ((char *) sharedInfo);

  /* Now we open again with the correct length */
  shmid = shmget (MY_SHM_KEY, sizeof (SharedInfo) + sizeof (LONG) * width,
    0600);
  if (shmid < 0)
  {
    fprintf (f, "Could not get shared memory\n");
    exit (1);
  }

  sharedInfo = (SharedInfo *) shmat (shmid, 0, 0);
  if ((int) sharedInfo < 0)
  {
    fprintf (f, "Could not attach shared memory\n");
    exit (1);
  }

  signal (SIGINT, TermHandler);		/* catch CTRL-C */
  signal (SIGTERM, TermHandler);	/* catch kill -TERM */
  signal (SIGHANDSHAKE, HandshakeHandler);
  ready = True;

  while (True)
  {
    if (ready)
    {
      while (sharedInfo->start == False)
        WaitForAck ();
      fprintf (f, "Received start command\n");
      sharedInfo->startACK = True;
      kill (parent, SIGHANDSHAKE);
      while (sharedInfo->start == True)
        WaitForAck ();
      /* Cancel any lines that might have been ready */
      sharedInfo->ready = False;
      ready = False;
      fprintf (f, "Acknowledged start command\n");
      sharedInfo->startACK = False;
    }
    else
    {
      fprintf (f, "Start calculation\n");
      memcpy (&pic, &sharedInfo->pic, sizeof (pic));
      pic.imC = pic.im1;
      current.reStep = (pic.re2 - pic.re1) / pic.width;
      current.imStep = (pic.im2 - pic.im1) / pic.height;
      timeout = False;
      signal (SIGALRM, AlarmHandler);
      alarm (1);
      iterations = 0;
      second.it_interval.tv_sec = 1;
      second.it_interval.tv_usec = 0;
      second.it_value.tv_sec = 1;
      second.it_value.tv_usec = 0;
      setitimer (ITIMER_VIRTUAL, &second, 0);
      signal (SIGVTALRM, VTAlarmHandler);

      for (current.row = 0; !ready && current.row < pic.height; current.row++)
      {
        pic.reC = pic.re1;
        for (current.column = 0; current.column < pic.width; current.column++)
        {
	  ((LONG *) &sharedInfo [1]) [current.column] = Iterate (&pic);
          pic.reC += current.reStep;
	  if (timeout)
	  {
            fprintf (f, "Handling timeout\n");
            timeout = False;
            signal (SIGALRM, AlarmHandler);
            alarm (1);
	    if (sharedInfo->start)
	    {
	      ready = True;
	      break;		/* exit loop */
	    }
	  }
        }

	/* One line is finished */
	if (!ready)
	{
	  sharedInfo->ready = True;
	  kill (parent, SIGHANDSHAKE);
	  while (sharedInfo->readyACK == False)
	    if (sharedInfo->start)
	    {
	      ready = True;
	      break;
	    }
	    else
	      WaitForAck ();
	  sharedInfo->ready = False;
	  kill (parent, SIGHANDSHAKE);
	  while (sharedInfo->readyACK == True)
	    if (sharedInfo->start)
	    {
	      ready = True;
	      break;
	    }
	    else
	      WaitForAck ();
          pic.imC += current.imStep;
          fprintf (f, "One line completed\n");
        }
      } /* for row */
      ready = True;
    } /* if (ready) */
  }

  return 0;			/* never reached */
} /* main */

