/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#include <math.h>
#include <stdlib.h>			/* strtoul */
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#include <Xlib.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Shell.h>

#include <Xaw3d/Reports.h>
#include <Xaw3d/Simple.h>
#include <Xaw3d/Viewport.h>

#include "mandel.h"
#include "mdefs.h"			/* portability */
#include "dialog.h"
#include "EditField.h"

#ifndef PI
#define PI M_PI
#endif


/* Titles of the main window */

#define PROGNAME "Mandel"
#define PROGCALC "Mandel (calculating)"


Image		pic;
ScreenInfo	mscreen;
SharedInfo	*sharedInfo;

Rubberband	rubberband;

struct
{
  Image		*stack;
  int		level;
} stack;

Boolean		newPicture;




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

static void HandshakeHandler (int sig)
{
  signal (sig, HandshakeHandler);
}




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void SaveToStack (void)
{
  Image *tempstack;

  stack.level++;
  tempstack = (Image *) realloc (stack.stack, stack.level * sizeof(Image));
  if (tempstack == 0)
  {
    Debug (0, "Out of memory\n");
    Exit (1);
  }
  Debug (1, "Image saved to stack, level %d\n", stack.level);
  stack.stack = tempstack;
  memcpy (&stack.stack [stack.level - 1], &pic, sizeof(pic));
} /* SaveToStack */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void RestoreFromStack (void)
{
  if (stack.level > 1)
  {
    stack.level--;
    memcpy (&pic, &stack.stack [stack.level], sizeof(pic));
    /* FIXME look wether we can load the picture from file */
    newPicture = True;
    pic.row = 0;
    Debug (1, "Image fetched from stack. New level %d\n", stack.level);
  }
} /* RestoreFromStack */




/*---------------------------------------------------------------------------
 *
 * Depth 8 only.
 *
 *---------------------------------------------------------------------------*/

void GetColormap (XColor *colors)
{
  int i;

  for (i = 0; i < 256; i++)
    colors [i].pixel = i;

  XQueryColors (XtDisplay (mscreen.toplevel), mscreen.colormap, colors, 256);
} /* GetColormap */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void EnterWindowCB (Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont)
{
  XSetWindowColormap (XtDisplay (w), XtWindow (w), mscreen.colormap);
} /* EnterWindowCB */




/*---------------------------------------------------------------------------
 *
 * Sets a colormap. For depth 8 only.
 *
 *---------------------------------------------------------------------------*/

void SetColormap (Widget w)
{
#if 1
  XColor	color [256];
  int		i;
  double	theta;
  double	saturation;
  int		intensity;

  mscreen.colormap = XCreateColormap (XtDisplay (w), XtWindow (w),
    DefaultVisualOfScreen (XtScreen (w)), AllocAll);
  /* Here could user defined colormaps be loaded */
  color [0].pixel = 0;
  color [0].red = 0;
  color [0].green = 0;
  color [0].blue = 0;
  color [0].flags = DoRed | DoGreen | DoBlue;
  for (i = 1; i < 256; i++)
  {
    /* Take the count as a HSI value, with H changing fastest.
       Say I 0 bit, S 1 bit, H 6 bit. */
    theta = 2 * PI * ((256 + 80 - i) & 127) / 128.0;
    saturation = ((((256 - i) >> 7) & 1) + 1) / 2.0;
    intensity = 2;
    color [i].pixel = i;
    color [i].red = (int) floor (intensity * 16383.0 * (1 + saturation * sin (theta - 2 * PI / 3)));
    color [i].green = (int) floor (intensity * 16383.0 * (1 + saturation * sin (theta)));
    color [i].blue = (int) floor (intensity * 16383.0 * (1 + saturation * sin (theta + 2 * PI / 3)));
    color [i].flags = DoRed | DoGreen | DoBlue;
  }
  XStoreColors (XtDisplay (w), mscreen.colormap, color, 256);
  XSetWindowColormap (XtDisplay (w), XtWindow (w), mscreen.colormap);
#else
  XColor	color;
  int		i;

  mscreen.colormap = DefaultColormapOfScreen (XtScreen (w));
  /* Here could user defined colormaps be loaded */
  for (i = 0; i < 256; i++)
  {
    color.pixel = i;
    color.red = (255 - i) << 8;
    color.green = (255 - i) << 8;
    color.blue = (255 - i) << 8;
    color.flags = DoRed | DoGreen | DoBlue;
    XAllocColor (XtDisplay (w), mscreen.colormap, &color);
  }
#endif
} /* SetColormap */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

int PixelColor (LONG count, int depth)
{
  int		r, g, b;
  double	theta;
  double	saturation;
  int		intensity;

  if (count == pic.maxc)
    return 0;
  else
    switch (depth)
    {
      case 8:
        return 255 - count % 255;
      break;
  
      case 15:
        /* color weight 5-5-5 */
	/* Take the count as a HSI value, with H changing fastest.
	   Say I 4 bit, S 5 bit, H 6 bit. */
	count = 0x8000 - (count & 0x7fff);
	/* The offset to count 'rotates' the colors. */
	theta = 2 * PI * ((count + 40) & 63) / 64.0;
	saturation = (((count >> 6) & 31) + 1) / 32.0;
	intensity = (count >> 11) & 15;
	r = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta - 2 * PI / 3)));
	g = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta)));
	b = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta + 2 * PI / 3)));
	return (r << 10) + (g << 5) + b;
      break;
  
      case 16:
	/* Take the count as a HSI value, with H changing fastest.
	   Say I 5 bit, S 5 bit, H 6 bit. */
	count = 0x10000 - (count & 0xffff);
	/* The offset to count 'rotates' the colors. */
	theta = 2 * PI * ((count + 40) & 63) / 64.0;
	saturation = (((count >> 6) & 31) + 1) / 32.0;
	intensity = (count >> 11) & 31;
	if (mscreen.weight565)
	{
	  r = (int) floor (intensity * 0.5 * (1 + saturation * sin (theta - 2 * PI / 3)));
	  g = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta)));
	  b = (int) floor (intensity * 0.5 * (1 + saturation * sin (theta + 2 * PI / 3)));
	  return (r << 11) + (g << 5) + b;
	}
	else
	{
	  r = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta - 2 * PI / 3)));
	  g = (int) floor (intensity * 0.5 * (1 + saturation * sin (theta)));
	  b = (int) floor (intensity * 0.5 * (1 + saturation * sin (theta + 2 * PI / 3)));
	  return (r << 10) + (g << 5) + b;
	}
      break;
  
      case 24:
        /* color weight 8-8-8 */
	/* Take the count as a HSI value, with H changing fastest.
	   Say I 7 bit, S 8 bit, H 9 bit. */
	count = 0x1000000 - (count & 0xffffff);
	/* The offset to count 'rotates' the colors. */
	theta = 2 * PI * ((count + 320) & 511) / 512.0;
	saturation = (((count >> 9) & 255) + 1) / 256.0;
	intensity = (count >> 17) & 127;
	r = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta - 2 * PI / 3)));
	g = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta)));
	b = (int) floor (intensity * 1.0 * (1 + saturation * sin (theta + 2 * PI / 3)));
	return (r << 16) + (g << 8) + b;
      break;
  
      default:
        Debug (0, "Don't know how to handle depth %d\n", depth);
        Exit (1);
    } /* switch */

  return 0;			/* Never reached */
} /* PixelColor */




/*---------------------------------------------------------------------------
 *
 * Depth 16 only.
 *
 *---------------------------------------------------------------------------*/

void DetectColorWeight (Display *display, Screen *screen)
{
  XColor	color;

  color.pixel = 31 << 5;
  XQueryColor (display, DefaultColormapOfScreen (screen), &color);
  mscreen.weight565 = (color.green < 32768);
} /* DetectColorWeight */




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

int main (int argc, char **argv)
{
  XtAppContext	appContext;
  Widget	canvas;
  Screen	*screen;
  GC		gc;
  XEvent	event;
  Window	w;
  int		column;

  pic.re1 = -2.5;
  pic.re2 = 1.5;
  pic.im1 = 1.5;
  pic.im2 = -1.5;
  pic.reP = 0.0;
  pic.imP = 0.0;
  pic.reJ = (pic.re2 + pic.re1) / 2;
  pic.imJ = (pic.im2 + pic.im1) / 2;
  pic.bailout = 4.0;
  pic.width = 240;
  pic.height = 180;
  pic.maxc = 100;
  pic.julia = False;

  child = 0;

  mscreen.toplevel = XtOpenApplication (&appContext, "Mandel", 0, 0,
    &argc, argv, 0, applicationShellWidgetClass, 0, 0);
  XtVaSetValues (mscreen.toplevel, XtNtitle, PROGNAME, 0);

  canvas = XtVaCreateManagedWidget ("canvas", viewportWidgetClass,
    mscreen.toplevel, 0);
  mscreen.draw = XtVaCreateManagedWidget ("draw", simpleWidgetClass, canvas,
    XtNwidth, pic.width, XtNheight, pic.height, 0);
  XtAddCallback (canvas, XtNreportCallback, ResizeCB, (XtPointer) mscreen.draw);

  XtRealizeWidget (mscreen.toplevel);

  /* Create dialog boxes */
  AboutInit (mscreen.toplevel);
  FileSelectInit (mscreen.toplevel);
  MessageInit (mscreen.toplevel);
  OptionInit (mscreen.toplevel);
  QuitInit (mscreen.toplevel);
  SizeInit (mscreen.toplevel);

  mscreen.display = XtDisplay (canvas);
  screen = XtScreen (canvas);
  gc = DefaultGCOfScreen (screen);
  mscreen.depth = DefaultDepthOfScreen (screen);
  if (mscreen.depth < 8)
  { 
    Debug (0, "Depths smaller than 8 are not supported\n");
    Exit (1);
  }
  else if (mscreen.depth == 8)
    SetColormap (mscreen.draw);
  else if (mscreen.depth == 16)
    DetectColorWeight (mscreen.display, screen);

  mscreen.bpp = (mscreen.depth + 7) >> 3;	/* e.g. depth 15, bpp 2 */
  if (mscreen.bpp == 3)
    mscreen.bpp = 4;
  w = XtWindow (mscreen.draw);

  signal (SIGHANDSHAKE, HandshakeHandler);
  signal (SIGTERM, Exit);
  signal (SIGINT, Exit);

  mscreen.image = 0;
  NewSize ();
  rubberband.SetScreen(screen);
  stack.stack = 0;
  stack.level = 0;
  SaveToStack ();

  XtAddEventHandler (mscreen.toplevel, EnterWindowMask, False,
    EnterWindowCB, 0);
  XtAddEventHandler (mscreen.draw, KeyPressMask, False, KeyPressCB, 0);
  XtAddEventHandler (mscreen.draw, ButtonPressMask, False, ButtonPressCB, 0);
  XtAddEventHandler (mscreen.draw, ButtonReleaseMask, False, ButtonReleaseCB, 0);
  XtAddEventHandler (mscreen.draw, PointerMotionMask, False, PointerMotionCB, 0);

  newPicture = True;
  pic.row = 0;

  while (True)
  {
    if (newPicture)
    {
      /* This is better than calling ExposeCB because the new image
         can be smaller than the old. */
      XClearWindow (mscreen.display, w);

      if (pic.row < pic.height)
      {
        XtVaSetValues (mscreen.toplevel, XtNtitle, PROGCALC, 0);
        memset (mscreen.data, 0, mscreen.bpp * pic.width * pic.height);
        NewPicture ();
      }
      else
      {
        XtVaSetValues (mscreen.toplevel, XtNtitle, PROGNAME, 0);
      }

      if (pic.row > 0)
        /* FIXME: Can we call ExposeCB instead? */
        XPutImage (mscreen.display, w, gc, mscreen.image, 0, 0,
          0, 0, pic.width, pic.row);

      newPicture = False;
    } /* if (newPicture) */

    if (XtAppPending (appContext))
    {
      XtAppNextEvent (appContext, &event);
      XtDispatchEvent (&event);
    }
    else if (sharedInfo->ready)
    {
      sharedInfo->readyACK = True;
      kill (child, SIGHANDSHAKE);
      for (column = 0; column < pic.width; column++)
        XPutPixel (mscreen.image, column, pic.row,
          PixelColor (((LONG *) &sharedInfo [1]) [column], mscreen.depth));

      while (sharedInfo->ready)
	WaitForAck ();
      sharedInfo->readyACK = False;
      kill (child, SIGHANDSHAKE);
      if (rubberband.Visible ())
      {
        /* delete rubberband, restore it afterwards */
	rubberband.Draw (mscreen.draw);
      }
      XPutImage (mscreen.display, w, gc, mscreen.image, 0, pic.row,
        0, pic.row, pic.width, 1);
      if (rubberband.Visible ())
      {
	rubberband.Draw (mscreen.draw);
      }
      pic.row++;
      if (pic.row >= pic.height)
        XtVaSetValues (mscreen.toplevel, XtNtitle, PROGNAME, 0);
    }
    else
      usleep (50);
  } /* while */
  return 0;

} /* main */

