/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 *                                                                           *
 * file: mandel.h                                                            *
 *                                                                           *
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef MANDEL_H
#define MANDEL_H

#include <unistd.h>
#include <Intrinsic.h>		/* Boolean */
#include "mdefs.h"
#include "Rubberband.h"


#define MY_SHM_KEY 0x3a3
#define SIGHANDSHAKE SIGUSR1

/* Where fractal images are stored */
#define frakDir "/home/hans/projekte/fractals/"


typedef struct
{
  DOUBLE        re1;
  DOUBLE        re2;
  DOUBLE        im1;
  DOUBLE        im2;
  DOUBLE        reP;                    /* re perturbation */
  DOUBLE        imP;                    /* im perturbation */
  DOUBLE        reC;                    /* current point */
  DOUBLE        imC;
  DOUBLE        reJ;                    /* point for julia fractal */
  DOUBLE        imJ;
  DOUBLE        bailout;                /* bailout value */
  int           width;                  /* width in pixels */
  int           height;                 /* height in pixels */
  int		row;			/* row currently calculated */
  LONG          maxc;                   /* max number of iterations */
  Boolean	julia;                  /* true if julia-set */
} Image;


typedef struct
{
  int		depth;		/* bits per pixel */
  int		bpp;		/* bytes per pixel */
  Display	*display;
  Colormap	colormap;
  XImage	*image;
  char          *data;		/* The image contents */
  Widget	toplevel;	/* Our toplevel widget */
  Widget	draw;		/* Widget that contains the image */
  Boolean	weight565;      /* true if green has 6 bits in 64 K colors */
} ScreenInfo;

typedef struct
{
  /* The structure of the shared memory */
  Image		pic;			/* image parameters */
  Boolean	start;			/* written by master */
  Boolean	startACK;		/* written by child */
  Boolean	ready;			/* written by child */
  Boolean	readyACK;		/* written by master */
} SharedInfo;


extern Boolean		newPicture;
extern Image		pic;
extern Image		readpic;
extern ScreenInfo	mscreen;
extern SharedInfo	*sharedInfo;
extern Rubberband	rubberband;
extern pid_t		child;


extern void Debug (int level, char *format, ...);
extern void Shutdown (void);
extern void Exit (int status);
extern void SaveToStack (void);
extern void RestoreFromStack (void);
extern void GetColormap (XColor *colormap);
extern LONG Iterate (Image *image);
extern void CreateChild (void);
extern void WaitForAck (void);
extern void NewPicture (void);

/* ppm.c */
extern Boolean WriteImage (const char *filename);
extern Boolean OpenImage (const char *filename);
extern Boolean ReadImage (const char *filename);
extern Boolean GetImage (const char *filename);

/* callback.c */
extern void KeyPressCB (Widget w, XtPointer clientData, XEvent *event,
  Boolean *cont);
extern void ButtonPressCB (Widget w, XtPointer clientData, XEvent *event,
  Boolean *cont);
extern void ButtonReleaseCB (Widget w, XtPointer clientData, XEvent *event,
  Boolean *cont);
extern void PointerMotionCB (Widget w, XtPointer clientData, XEvent *event,
    Boolean *cont);
extern void ExposeCB (Widget w, XtPointer clientData, XEvent *event, Boolean *cont);
extern void ResizeCB (Widget w, XtPointer clientData, XtPointer callData);

#endif /* MANDEL_H */

