#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw3d/Box.h>
#include <X11/Xaw3d/Command.h>
#include <X11/Xaw3d/Toggle.h>

#include "EditField.h"
#include "netchaos.h"


static Widget       sizeBox;
static Widget       sizeToggle0;
static Widget       sizeToggle1;
static Widget       sizeToggle2;
static Widget       sizeToggle3;
static Widget       sizeToggle4;
static Widget       sizeToggle5;
static EditField    sizeFieldX;
static EditField    sizeFieldY;

int            lastSizeNumber;
int            lastXSize;
int            lastYSize;




/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void NewSize()
{
    lastXSize = pic.width;
    lastYSize = pic.height;

    if (mscreen.image != 0)
    {
        XtRemoveEventHandler(mscreen.draw, ExposureMask, False, ExposeCB,
            mscreen.image);
        /* XDestroyImage also frees data */
        XDestroyImage(mscreen.image);
    }

    mscreen.data = (char *) malloc(mscreen.bpp * pic.width * pic.height);
    if (mscreen.data == 0)
    {
        Debug(0, "Not enough memory for this picture\n");
        Exit(1);
    }

    /* XYPixmap gives totally wrong colors. */
    mscreen.image = XCreateImage(mscreen.display,
        DefaultVisualOfScreen(XtScreen(mscreen.draw)), mscreen.depth,
        ZPixmap, 0, mscreen.data,
        pic.width, pic.height, 8, 0);

    if (mscreen.image == 0)
    {
        Debug(0, "Error creating XImage structure\n");
        Exit(1);
    }

    /* ExposureMask is a constant defined in X.h */
    XtAddEventHandler(mscreen.draw, ExposureMask, False, ExposeCB,
        mscreen.image);

    XtVaSetValues(mscreen.draw, XtNwidth, pic.width,
        XtNheight, pic.height, 0);

} /* NewSize */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

static void SizeCB(Widget w, XtPointer clientData, XtPointer callData)
{
    Image       newpic;
    Boolean     error = False;
    int         number;
    char        *s;

    XtPopdown(sizeBox);
    if ((Boolean) clientData != True)
        return;

    /* Fetch the new data and look for changes */
    newpic = pic;
    s = (char *) XawToggleGetCurrent(sizeToggle1);
    if (s == 0)
        number = lastSizeNumber;
    else
        number = s [strlen(s) - 1] - '0';

    if (number == 0)
    {
        error = error || !sizeFieldX.GetValue(XtNstring, &newpic.width);
        error = error || !sizeFieldY.GetValue(XtNstring, &newpic.height);
    }

    if (!error && (lastSizeNumber != number ||
            number == 0 && memcmp(&newpic, &pic, sizeof(pic)) != 0))
    {
        switch (number)
        {
            case 1:
                newpic.width = 240;
                newpic.height = 180;
            break;

            case 2:
                newpic.width = 320;
                newpic.height = 240;
            break;

            case 3:
                newpic.width = 640;
                newpic.height = 480;
            break;

            case 4:
                newpic.width = 800;
                newpic.height = 600;
            break;

            case 5:
                newpic.width = 1024;
                newpic.height = 768;
            break;
        } /* switch */

        lastSizeNumber = number;
        pic = newpic;
        NewSize();
        newPicture = True;
    }

} /* SizeCB */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void SizePopup(int x, int y)
{
    XtVaSetValues(sizeBox, XtNx, x, XtNy, y, 0);
    sizeFieldX.SetValue(XtNstring, &lastXSize);
    sizeFieldY.SetValue(XtNstring, &lastYSize);

    XtVaSetValues(sizeToggle0, XtNstate, lastSizeNumber == 0, 0);
    XtVaSetValues(sizeToggle1, XtNstate, lastSizeNumber == 1, 0);
    XtVaSetValues(sizeToggle2, XtNstate, lastSizeNumber == 2, 0);
    XtVaSetValues(sizeToggle3, XtNstate, lastSizeNumber == 3, 0);
    XtVaSetValues(sizeToggle4, XtNstate, lastSizeNumber == 4, 0);
    XtVaSetValues(sizeToggle5, XtNstate, lastSizeNumber == 5, 0);

    XtPopup(sizeBox, XtGrabExclusive);

} /* SizePopup */





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void SizeInit(Widget parent)
{
    Widget  sizeContainer;
    Widget  sizeToggleBox;
    Widget  sizeButtonBox;
    Widget  sizeButton1;
    Widget  sizeButton2;

    sizeBox = XtVaCreatePopupShell("sizebox", topLevelShellWidgetClass,
        parent, 0);
    sizeContainer = XtVaCreateManagedWidget("sizecontainer", boxWidgetClass,
        sizeBox, 0);
    sizeToggleBox = XtVaCreateManagedWidget("sizetogglebox", boxWidgetClass,
        sizeContainer, 0);
    sizeToggle1 = XtVaCreateManagedWidget("sizetoggle1", toggleWidgetClass,
        sizeToggleBox, 0);
    sizeToggle2 = XtVaCreateManagedWidget("sizetoggle2", toggleWidgetClass,
        sizeToggleBox, 0);
    sizeToggle3 = XtVaCreateManagedWidget("sizetoggle3", toggleWidgetClass,
        sizeToggleBox, 0);
    sizeToggle4 = XtVaCreateManagedWidget("sizetoggle4", toggleWidgetClass,
        sizeToggleBox, 0);
    sizeToggle5 = XtVaCreateManagedWidget("sizetoggle5", toggleWidgetClass,
        sizeToggleBox, 0);
    sizeToggle0 = XtVaCreateManagedWidget("sizetoggle0", toggleWidgetClass,
        sizeToggleBox, 0);

    XtVaSetValues(sizeToggle1, XtNradioGroup, sizeToggle2, 0);
    XtVaSetValues(sizeToggle2, XtNradioGroup, sizeToggle1, 0);
    XtVaSetValues(sizeToggle3, XtNradioGroup, sizeToggle1, 0);
    XtVaSetValues(sizeToggle4, XtNradioGroup, sizeToggle1, 0);
    XtVaSetValues(sizeToggle5, XtNradioGroup, sizeToggle1, 0);
    XtVaSetValues(sizeToggle0, XtNradioGroup, sizeToggle1, 0);

    sizeFieldX.Create(sizeContainer, 1, "size", EDT_ULONG);
    sizeFieldY.Create(sizeContainer, 2, "size", EDT_ULONG);

    sizeButtonBox = XtVaCreateManagedWidget("sizebuttonbox", boxWidgetClass,
        sizeContainer, 0);
    sizeButton1 = XtVaCreateManagedWidget("sizebutton1", commandWidgetClass,
        sizeButtonBox, 0);
    sizeButton2 = XtVaCreateManagedWidget("sizebutton2", commandWidgetClass,
        sizeButtonBox, 0);

    XtAddCallback(sizeButton1, XtNcallback, SizeCB, (caddr_t) True);
    XtAddCallback(sizeButton2, XtNcallback, SizeCB, (caddr_t) False);

    lastSizeNumber = 0;

} /* SizeInit */
