#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include <Xlib.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Shell.h>

#include <Xaw3d/Reports.h>
#include <Xaw3d/Simple.h>
#include <Xaw3d/Viewport.h>

int debugLevel = 8;
int width = 250;
int height = 160;
XImage *image = NULL;
char *data;
Widget toplevel;
Widget canvas;
Widget draw;

void Debug(int level, char *format, ...)
{
    va_list args;
    va_start(args, format);
    if (level <= debugLevel)
        vprintf(format, args);
    va_end(args);
} /* Debug */

void ExposeCB(Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
    XExposeEvent    *ev = (XExposeEvent *) event;

    Debug(8, "Expose\n");
    if (event == 0)
    {
        Debug(0, "Event pointer is 0 in ExposeCB\n");
        return;
    }

    if (clientData == 0)
    {
        Debug(0, "clientData is 0 in ExposeCB\n");
        return;
    }

    XPutImage(XtDisplay(w), XtWindow(w),  DefaultGCOfScreen(XtScreen(w)),
        (XImage *) clientData,
        ev->x, ev->y, ev->x, ev->y, ev->width, ev->height);

} /* ExposeCB */

void KeyPressCB(Widget w, XtPointer clientData, XEvent *event, Boolean *cont)
{
    char            buffer[80];
    KeySym          key;

    XLookupString((XKeyEvent *) event, buffer, sizeof(buffer), &key, 0);

    switch(key)
    {
        case 'q':
            exit(0);
        break;
        case 's':
            if (image != NULL)
            {
                XtRemoveEventHandler(draw, ExposureMask, False, ExposeCB, image);
                XDestroyImage(image);
            }
            width = 750 - width;
            height = 480 - height;
            data = (char *) malloc(4 * width * height);
            memset(data, 0xff0000, 4 * width * height);
            image = XCreateImage(XtDisplay(draw),
                    DefaultVisualOfScreen(XtScreen(draw)),
                    DefaultDepthOfScreen(XtScreen(draw)), ZPixmap, 0, data,
                                    width, height, 8, 0); 
            XtVaSetValues(draw, XtNwidth, width, XtNheight, height, 0);
            XtAddEventHandler(draw, ExposureMask, False, ExposeCB, image);
            Debug(0, "XClearWindow\n");
            XClearWindow(XtDisplay(draw), XtWindow(draw));
            XPutPixel(image, 10, 10, 0xffffff);
            XPutImage(XtDisplay(draw), XtWindow(draw),
                DefaultGCOfScreen(XtScreen(draw)),
                image, 0, 0, 0, 0, width, height);
        break;
    }
}

void ResizeCB(Widget w, XtPointer clientData, XtPointer callData)
{
    static int entryCount = 0;
#define cd ((XawPannerReport *) callData)

    if (++entryCount <= 1)
    {
        if (cd->changed & (XawPRCanvasWidth | XawPRCanvasHeight))
        {
            Boolean modified = False;

            Debug(8, "Resize\n");
            Debug(8, "%d %d %d %d\n",
                (int) cd->slider_width,
                (int) cd->slider_height,
                (int) cd->canvas_width,
                (int) cd->canvas_height);

            if (cd->canvas_width > width)
            {
                cd->canvas_width = width;
                modified = True;
            }

            if (cd->canvas_height > height)
            {
                cd->canvas_height = height;
                modified = True;
            }

            if (modified)
            {
                XtVaSetValues(toplevel, XtNwidth, cd->canvas_width,
                    XtNheight, cd->canvas_height, 0);
            }

            XtVaSetValues((Widget) clientData, XtNwidth, width,
                XtNheight, height, 0);
        }
    }

    --entryCount;

#undef cd

} /* ResizeCB */

int main(int argc, char **argv)
{
    XtAppContext    appContext;

    toplevel = XtOpenApplication(&appContext, "Mandel", 0, 0,
        &argc, argv, 0, applicationShellWidgetClass, 0, 0);

    canvas = XtVaCreateManagedWidget("canvas", viewportWidgetClass,
        toplevel, 0);
    draw = XtVaCreateManagedWidget("draw", simpleWidgetClass, canvas,
        XtNwidth, 250, XtNheight, 160, 0);
    XtAddCallback(canvas, XtNreportCallback, ResizeCB, (XtPointer) draw);

    XtAddEventHandler(draw, KeyPressMask, False, KeyPressCB, 0);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(appContext);

    return 0;       // Never reached

} /* main */
