/*
 * Implementation of
 *
 *  HISTORY
 *  Date        Author  Remarks
 *  1996-10-17  hjb     Reformatted
 *  1999-02-04  hjb     Include stdio.h for glibc2 compatibility
 *  1999-02-14  hjb     Re-introduced WaitForAck
 *  1999-02-19  hjb     Added SetDebugLevel
 *  1999-02-21  hjb     Moved WaitForAck to comm.c
 *  1999-03-10  hjb     New function GetTime
 */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include "netchaos.h"


static int debugLevel = 9;





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void *xmalloc(size_t size)
{
    void *p = malloc(size);
    if (p == NULL)
    {
        Debug(0, "Out of memory\n");
        Exit(1);
    }

    return p;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void *xrealloc(void *p, size_t size)
{
    void *q = realloc(p, size);
    if (q == NULL)
    {
        Debug(0, "Out of memory\n");
        Exit(1);
    }
    return q;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void *xcalloc(size_t size)
{
    void *p = xmalloc(size);
    memset(p, 0, size);
    return p;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void xfree(void *p)
{
    free(p);
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void GetTime(struct timeval& tv)
    if (gettimeofday(&tv, NULL) == -1)
    {
        tv.tv_sec = -1;
        tv.tv_usec = -1;
    }
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

int GetDebugLevel()
{
    return debugLevel;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void SetDebugLevel(int level)
{
    debugLevel = level;
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void SetDebugFile(const char *filename)
{
}





/*---------------------------------------------------------------------------
 *
 *
 *---------------------------------------------------------------------------*/

void Debug(int level, char *format, ...)
{
    va_list args;
    int     category = level >> 8;          /* Not used yet */

    va_start(args, format);

    level &= 255;
    if (level <= debugLevel)
        vprintf(format, args);

    va_end(args);

} /* Debug */
